@@ -75,6 +75,7 @@ typedef struct opkg_conf {
     char *lock_file;
     char *info_dir;
     char *status_file;
+    char *fields_filter; /* specific fields the user requests */
 
     unsigned int pfm;       /* package field mask */
 
@@ -120,6 +121,7 @@ typedef struct opkg_conf {
     int host_cache_dir;
     int verbose_status_file;
     int compress_list_files;
+    int short_description;
 
     /* ssl options: used only when opkg is configured with '--enable-curl',
      * otherwise always NULL or 0.
