@@ -389,10 +389,8 @@ int pkg_merge(pkg_t * oldpkg, pkg_t * newpkg)
         oldpkg->tmp_unpack_dir = xstrdup(newpkg->tmp_unpack_dir);
     if (!oldpkg->md5sum)
         oldpkg->md5sum = xstrdup(newpkg->md5sum);
-#if defined HAVE_SHA256
     if (!oldpkg->sha256sum)
         oldpkg->sha256sum = xstrdup(newpkg->sha256sum);
-#endif
     if (!oldpkg->size)
         oldpkg->size = newpkg->size;
     if (!oldpkg->installed_size)
@@ -546,13 +544,21 @@ static const char *pkg_state_status_to_str(pkg_state_status_t ss)
     return "<STATE_STATUS_UNKNOWN>";
 }
 
-static void pkg_formatted_field(FILE * fp, pkg_t * pkg, const char *field)
+static int should_include_field(const char *field, const char *fields_filter)
+{
+   return field && (!fields_filter || strstr(fields_filter, field));
+}
+
+static void pkg_formatted_field(FILE * fp, pkg_t * pkg, const char *field, const char *fields_filter)
 {
     unsigned int i, j;
     char *str;
     unsigned int depends_count = pkg->pre_depends_count + pkg->depends_count
         + pkg->recommends_count + pkg->suggests_count;
 
+    if (!should_include_field(field, fields_filter)) {
+       return;
+    }
     if (strlen(field) < PKG_MINIMUM_FIELD_NAME_LEN) {
         goto UNKNOWN_FMT_FIELD;
     }
@@ -624,7 +630,15 @@ static void pkg_formatted_field(FILE * fp, pkg_t * pkg, const char *field)
             }
         } else if (strcasecmp(field, "Description") == 0) {
             if (pkg->description) {
-                fprintf(fp, "Description: %s\n", pkg->description);
+                const char* first_line_end = strchr(pkg->description, '\n');
+                if (opkg_config->short_description && first_line_end) {
+                    fprintf(fp, "Description: %.*s\n",
+                            (int)(first_line_end - pkg->description),
+                            pkg->description);
+                }
+                else {
+                    fprintf(fp, "Description: %s\n", pkg->description);
+                }
             }
         } else {
             goto UNKNOWN_FMT_FIELD;
@@ -738,12 +752,10 @@ static void pkg_formatted_field(FILE * fp, pkg_t * pkg, const char *field)
             if (pkg->section) {
                 fprintf(fp, "Section: %s\n", pkg->section);
             }
-#if defined HAVE_SHA256
         } else if (strcasecmp(field, "SHA256sum") == 0) {
             if (pkg->sha256sum) {
                 fprintf(fp, "SHA256sum: %s\n", pkg->sha256sum);
             }
-#endif
         } else if (strcasecmp(field, "Size") == 0) {
             if (pkg->size) {
                 fprintf(fp, "Size: %ld\n", pkg->size);
@@ -807,7 +819,7 @@ static void pkg_formatted_field(FILE * fp, pkg_t * pkg, const char *field)
     opkg_msg(ERROR, "Internal error: field=%s\n", field);
 }
 
-static void pkg_formatted_userfields(FILE *fp, pkg_t *pkg)
+static void pkg_formatted_userfields(FILE *fp, pkg_t *pkg, const char *fields_filter)
 {
     nv_pair_list_elt_t *iter;
 
@@ -818,9 +830,8 @@ static void pkg_formatted_userfields(FILE *fp, pkg_t *pkg)
                 iter = nv_pair_list_next(&pkg->userfields, iter)) {
         nv_pair_t *uf = (nv_pair_t *)iter->data;
 
-        if (uf->name && uf->value) {
-            fprintf(fp, "%s: %s\n", ((nv_pair_t *)iter->data)->name,
-                    ((nv_pair_t *)iter->data)->value);
+        if (should_include_field(uf->name, fields_filter) && uf->value) {
+            fprintf(fp, "%s: %s\n", uf->name, uf->value);
         }
     }
 }
@@ -839,32 +850,32 @@ pkg_state_status_t pkg_state_status_from_str(const char *str)
     return SS_NOT_INSTALLED;
 }
 
-void pkg_formatted_info(FILE * fp, pkg_t * pkg)
+void pkg_formatted_info(FILE * fp, pkg_t * pkg, const char *fields_filter)
 {
-    pkg_formatted_field(fp, pkg, "Package");
-    pkg_formatted_field(fp, pkg, "Version");
-    pkg_formatted_field(fp, pkg, "Depends");
-    pkg_formatted_field(fp, pkg, "Recommends");
-    pkg_formatted_field(fp, pkg, "Suggests");
-    pkg_formatted_field(fp, pkg, "Provides");
-    pkg_formatted_field(fp, pkg, "Replaces");
-    pkg_formatted_field(fp, pkg, "Conflicts");
-    pkg_formatted_field(fp, pkg, "Status");
-    pkg_formatted_field(fp, pkg, "Section");
-    pkg_formatted_field(fp, pkg, "Essential");
-    pkg_formatted_field(fp, pkg, "Architecture");
-    pkg_formatted_field(fp, pkg, "Maintainer");
-    pkg_formatted_field(fp, pkg, "MD5sum");
-    pkg_formatted_field(fp, pkg, "Size");
-    pkg_formatted_field(fp, pkg, "Filename");
-    pkg_formatted_field(fp, pkg, "Conffiles");
-    pkg_formatted_field(fp, pkg, "Source");
-    pkg_formatted_field(fp, pkg, "Description");
-    pkg_formatted_field(fp, pkg, "Installed-Size");
-    pkg_formatted_field(fp, pkg, "Installed-Time");
-    pkg_formatted_field(fp, pkg, "Tags");
+    pkg_formatted_field(fp, pkg, "Package", NULL);
+    pkg_formatted_field(fp, pkg, "Version", fields_filter);
+    pkg_formatted_field(fp, pkg, "Depends", fields_filter);
+    pkg_formatted_field(fp, pkg, "Recommends", fields_filter);
+    pkg_formatted_field(fp, pkg, "Suggests", fields_filter);
+    pkg_formatted_field(fp, pkg, "Provides", fields_filter);
+    pkg_formatted_field(fp, pkg, "Replaces", fields_filter);
+    pkg_formatted_field(fp, pkg, "Conflicts", fields_filter);
+    pkg_formatted_field(fp, pkg, "Status", fields_filter);
+    pkg_formatted_field(fp, pkg, "Section", fields_filter);
+    pkg_formatted_field(fp, pkg, "Essential", fields_filter);
+    pkg_formatted_field(fp, pkg, "Architecture", fields_filter);
+    pkg_formatted_field(fp, pkg, "Maintainer", fields_filter);
+    pkg_formatted_field(fp, pkg, "MD5sum", fields_filter);
+    pkg_formatted_field(fp, pkg, "Size", fields_filter);
+    pkg_formatted_field(fp, pkg, "Filename", fields_filter);
+    pkg_formatted_field(fp, pkg, "Conffiles", fields_filter);
+    pkg_formatted_field(fp, pkg, "Source", fields_filter);
+    pkg_formatted_field(fp, pkg, "Description", fields_filter);
+    pkg_formatted_field(fp, pkg, "Installed-Size", fields_filter);
+    pkg_formatted_field(fp, pkg, "Installed-Time", fields_filter);
+    pkg_formatted_field(fp, pkg, "Tags", fields_filter);
     if (opkg_config->verbose_status_file) {
-        pkg_formatted_userfields(fp, pkg);
+        pkg_formatted_userfields(fp, pkg, fields_filter);
     }
     fputs("\n", fp);
 }
@@ -875,36 +886,36 @@ void pkg_print_status(pkg_t * pkg, FILE * file)
         return;
     }
 
-    pkg_formatted_field(file, pkg, "Package");
-    pkg_formatted_field(file, pkg, "Version");
-    pkg_formatted_field(file, pkg, "Depends");
-    pkg_formatted_field(file, pkg, "Recommends");
-    pkg_formatted_field(file, pkg, "Suggests");
-    pkg_formatted_field(file, pkg, "Provides");
-    pkg_formatted_field(file, pkg, "Replaces");
-    pkg_formatted_field(file, pkg, "Conflicts");
-    pkg_formatted_field(file, pkg, "Status");
+    pkg_formatted_field(file, pkg, "Package", NULL);
+    pkg_formatted_field(file, pkg, "Version", NULL);
+    pkg_formatted_field(file, pkg, "Depends", NULL);
+    pkg_formatted_field(file, pkg, "Recommends", NULL);
+    pkg_formatted_field(file, pkg, "Suggests", NULL);
+    pkg_formatted_field(file, pkg, "Provides", NULL);
+    pkg_formatted_field(file, pkg, "Replaces", NULL);
+    pkg_formatted_field(file, pkg, "Conflicts", NULL);
+    pkg_formatted_field(file, pkg, "Status", NULL);
     if (opkg_config->verbose_status_file) {
-        pkg_formatted_field(file, pkg, "Section");
+        pkg_formatted_field(file, pkg, "Section", NULL);
     }
-    pkg_formatted_field(file, pkg, "Essential");
-    pkg_formatted_field(file, pkg, "Architecture");
+    pkg_formatted_field(file, pkg, "Essential", NULL);
+    pkg_formatted_field(file, pkg, "Architecture", NULL);
     if (opkg_config->verbose_status_file) {
-        pkg_formatted_field(file, pkg, "Maintainer");
-        pkg_formatted_field(file, pkg, "MD5sum");
-        pkg_formatted_field(file, pkg, "Size");
-        pkg_formatted_field(file, pkg, "Filename");
+        pkg_formatted_field(file, pkg, "Maintainer", NULL);
+        pkg_formatted_field(file, pkg, "MD5sum", NULL);
+        pkg_formatted_field(file, pkg, "Size", NULL);
+        pkg_formatted_field(file, pkg, "Filename", NULL);
     }
-    pkg_formatted_field(file, pkg, "Conffiles");
+    pkg_formatted_field(file, pkg, "Conffiles", NULL);
     if (opkg_config->verbose_status_file) {
-        pkg_formatted_field(file, pkg, "Source");
-        pkg_formatted_field(file, pkg, "Description");
+        pkg_formatted_field(file, pkg, "Source", NULL);
+        pkg_formatted_field(file, pkg, "Description", NULL);
     }
-    pkg_formatted_field(file, pkg, "Installed-Size");
-    pkg_formatted_field(file, pkg, "Installed-Time");
-    pkg_formatted_field(file, pkg, "Auto-Installed");
+    pkg_formatted_field(file, pkg, "Installed-Size", NULL);
+    pkg_formatted_field(file, pkg, "Installed-Time", NULL);
+    pkg_formatted_field(file, pkg, "Auto-Installed", NULL);
     if (opkg_config->verbose_status_file) {
-        pkg_formatted_userfields(file, pkg);
+        pkg_formatted_userfields(file, pkg, NULL);
     }
     fputs("\n", file);
 }
@@ -1207,11 +1218,10 @@ file_list_t *pkg_get_installed_files(pkg_t * pkg)
                 // already contains root_dir as header -> ABSOLUTE
                 sprintf_alloc(&installed_file_name, "%s", file_name);
             }
-            if (xlstat(installed_file_name, &file_stat) == 0) {
+            if (!mode && xlstat(installed_file_name, &file_stat) == 0)
                 mode = file_stat.st_mode;
-                if (S_ISLNK(mode))
-                    link_target = readlink_buf = file_readlink_alloc(installed_file_name);
-            }
+            if (!link_target && S_ISLNK(mode))
+                link_target = readlink_buf = file_readlink_alloc(installed_file_name);
         }
         file_list_append(pkg->installed_files, installed_file_name, mode, link_target);
         free(installed_file_name);
@@ -1406,7 +1416,34 @@ static void pkg_write_filelist_helper(const char *key, void *entry_,
     struct pkg_write_filelist_data *data = data_;
     pkg_t *entry = entry_;
     if (entry == data->pkg) {
-        fprintf(data->stream, "%s\n", key);
+        char *installed_file_name;
+        struct stat file_stat;
+        mode_t mode = 0;
+        char *link_target = NULL;
+        int unmatched_offline_root = opkg_config->offline_root
+                && !str_starts_with(key, opkg_config->offline_root);
+        if (unmatched_offline_root) {
+            sprintf_alloc(&installed_file_name, "%s%s",
+                          opkg_config->offline_root, key);
+        } else {
+            // already contains root_dir as header -> ABSOLUTE
+            sprintf_alloc(&installed_file_name, "%s", key);
+        }
+        if (xlstat(installed_file_name, &file_stat) == 0) {
+            mode = file_stat.st_mode;
+            if (S_ISLNK(mode))
+                link_target = file_readlink_alloc(installed_file_name);
+        }
+
+        if (link_target)
+            fprintf(data->stream, "%s\t%#03o\t%s\n", key, (unsigned int)mode, link_target);
+        else if (mode)
+            fprintf(data->stream, "%s\t%#03o\n", key, (unsigned int)mode);
+        else
+            fprintf(data->stream, "%s\n", key);
+
+        free(link_target);
+        free(installed_file_name);
     }
 }
 
@@ -1469,22 +1506,28 @@ int pkg_verify(pkg_t * pkg)
     int err;
     char *local_sig_filename = NULL;
 
-    /* Silently fail verification if the package doesn't exist locally as the
-     * caller may be about to download it. */
+    /* Exit if the package doesn't exist locally as the caller may be about to
+     * download it. */
     if (!file_exists(pkg->local_filename))
-        return -1;
-
-    if (pkg->md5sum) {
-        err = opkg_verify_md5sum(pkg->local_filename, pkg->md5sum);
-        if (err)
-            goto fail;
-    }
+        return 1;
 
+#ifdef HAVE_SHA256
     if (pkg->sha256sum) {
         err = opkg_verify_sha256sum(pkg->local_filename, pkg->sha256sum);
         if (err)
             goto fail;
     }
+    else
+#endif
+    if (pkg->md5sum) {
+        err = opkg_verify_md5sum(pkg->local_filename, pkg->md5sum);
+        if (err)
+            goto fail;
+    } else if (!opkg_config->force_checksum) {
+         opkg_msg(ERROR, "Checksum is either missing or unsupported on opkg. To bypass verification "
+                  "use --force-checksum. Aborting \n");
+         return -1;
+    }
 
     if (opkg_config->check_pkg_signature) {
         local_sig_filename = pkg_download_signature(pkg);
