@@ -19,13 +19,14 @@
 #include "config.h"
 
 #include <stdio.h>
-#include <malloc.h>
+#include <stdlib.h>
 #include <fnmatch.h>
 
 #include "hash_table.h"
 #include "release.h"
 #include "pkg.h"
 #include "opkg_message.h"
+#include "opkg_archive.h"
 #include "pkg_vec.h"
 #include "pkg_hash.h"
 #include "parse_util.h"
@@ -65,15 +66,41 @@ static int pkg_hash_add_from_file(const char *file_name, pkg_src_t * src,
                            pkg_dest_t * dest, int is_status_file)
 {
     pkg_t *pkg;
-    FILE *fp;
-    char *buf;
+    FILE *fp = NULL;
+    char *buf = NULL, *bp = NULL;
     const size_t len = 4096;
     int ret = 0;
 
-    fp = fopen(file_name, "r");
-    if (fp == NULL) {
-        opkg_perror(ERROR, "Failed to open %s", file_name);
-        return -1;
+    if (opkg_config->compress_list_files  && !is_status_file) {
+        struct opkg_ar *ar;
+        size_t size;
+
+        ar = ar_open_compressed_file(file_name);
+        if (!ar)
+            return -1;
+
+        FILE *mfp = open_memstream(&bp, &size);
+
+        if (ar_copy_to_stream(ar, mfp) < 0) {
+            opkg_perror(ERROR, "Failed to open %s", file_name);
+            ret = -1;
+            goto cleanup;
+        }
+        fclose(mfp);
+
+        fp = fmemopen(bp, size, "r");
+        if (fp == NULL) {
+            opkg_perror(ERROR, "Failed to open memory buffer: %s\n", strerror(errno));
+            ret = -1;
+            goto cleanup;
+        }
+    } else {
+        fp = fopen(file_name, "r");
+        if (fp == NULL) {
+            opkg_perror(ERROR, "Failed to open %s", file_name);
+            ret = -1;
+            goto cleanup;
+        }
     }
 
     /* Remove UTF-8 BOM if present */
@@ -129,8 +156,11 @@ static int pkg_hash_add_from_file(const char *file_name, pkg_src_t * src,
 
     } while (!feof(fp));
 
+cleanup:
     free(buf);
-    fclose(fp);
+    if (fp)
+        fclose(fp);
+    free(bp);
 
     return ret;
 }
@@ -191,8 +221,8 @@ int pkg_hash_load_feeds(void)
 
         src = (pkg_src_t *) iter->data;
 
-        sprintf_alloc(&list_file, "%s/%s", opkg_config->lists_dir, src->name);
-
+        sprintf_alloc(&list_file, "%s/%s%s", opkg_config->lists_dir, src->name,
+                      opkg_config->compress_list_files ? ".gz" : "");
         if (file_exists(list_file)) {
             unsigned int i;
             release_t *release = release_new();
@@ -229,7 +259,8 @@ int pkg_hash_load_feeds(void)
 
         src = (pkg_src_t *) iter->data;
 
-        sprintf_alloc(&list_file, "%s/%s", opkg_config->lists_dir, src->name);
+        sprintf_alloc(&list_file, "%s/%s%s", opkg_config->lists_dir, src->name,
+                      opkg_config->compress_list_files ? ".gz" : "" );
 
         if (file_exists(list_file)) {
             r = pkg_hash_add_from_file(list_file, src, NULL, 0);
@@ -603,10 +634,12 @@ pkg_t *pkg_hash_fetch_best_installation_candidate_by_name(const char *name)
     return pkg_hash_fetch_best_installation_candidate(apkg, NULL, NULL, 0, 0);
 }
 
-pkg_t *pkg_hash_fetch_by_name_version(const char *pkg_name, const char *version)
+pkg_t *pkg_hash_fetch_by_name_version_arch(const char *pkg_name,
+                                           const char *version,
+                                           const char *arch)
 {
     pkg_vec_t *vec;
-    unsigned int i;
+    long i;
     char *version_str = NULL;
 
     vec = pkg_vec_fetch_by_name(pkg_name);
@@ -618,7 +651,9 @@ pkg_t *pkg_hash_fetch_by_name_version(const char *pkg_name, const char *version)
     // This is so that libsolv choses the correct package for reinstall
     for (i = vec->len-1; i >= 0; i--) {
         version_str = pkg_version_str_alloc(vec->pkgs[i]);
-        if (strcmp(version_str, version) == 0) {
+        int is_match = (strcmp(version_str, version) == 0)
+                && (strcmp(vec->pkgs[i]->architecture, arch) == 0);
+        if (is_match) {
             free(version_str);
             break;
         }
@@ -872,9 +907,9 @@ void file_hash_set_file_owner(const char *file_name, pkg_t * owning_pkg)
     hash_table_insert(&opkg_config->file_hash, file_name, owning_pkg);
 
     if (old_owning_pkg) {
-        pkg_get_installed_files(old_owning_pkg);
+        if (!old_owning_pkg->installed_files)
+            pkg_get_installed_files(old_owning_pkg);
         file_list_remove_elt(old_owning_pkg->installed_files, file_name);
-        pkg_free_installed_files(old_owning_pkg);
 
         /* mark this package to have its filelist written */
         old_owning_pkg->state_flag |= SF_FILELIST_CHANGED;
