@@ -524,6 +524,12 @@ static void populate_installed_repo(libsolv_solver_t *libsolv_solver)
         if (opkg_config->force_depends)
             queue_push2(&libsolv_solver->solver_jobs, SOLVER_SOLVABLE
                         | SOLVER_WEAKENDEPS, solvable_id);
+
+        if (pkg->essential && !opkg_config->force_removal_of_essential_packages) {
+            Id essential_pkg_id = pool_str2id(libsolv_solver->pool, pkg->name, 1);
+            queue_push2(&libsolv_solver->solver_jobs, SOLVER_SOLVABLE_NAME
+                        | SOLVER_INSTALL | SOLVER_ESSENTIAL, essential_pkg_id);
+        }
     }
 
     pkg_vec_free(installed_pkgs);
@@ -837,8 +843,9 @@ static int libsolv_solver_transaction_preamble(libsolv_solver_t *libsolv_solver,
 
         const char *pkg_name = pool_id2str(libsolv_solver->pool, solvable->name);
         const char *evr = pool_id2str(libsolv_solver->pool, solvable->evr);
+        const char *arch = pool_id2str(libsolv_solver->pool, solvable->arch);
 
-        pkg = pkg_hash_fetch_by_name_version(pkg_name, evr);
+        pkg = pkg_hash_fetch_by_name_version_arch(pkg_name, evr, arch);
         pkg_vec_insert(pkgs, pkg);
 
         if (!no_action && pkg->local_filename == NULL &&
@@ -887,14 +894,20 @@ static int libsolv_solver_execute_transaction(libsolv_solver_t *libsolv_solver)
             switch (typeId) {
             case SOLVER_TRANSACTION_ERASE:
                 ret = opkg_remove_pkg(pkg);
+                if (ret) {
+                    err = -1;
+                    goto CLEANUP;
+                }
                 break;
             case SOLVER_TRANSACTION_OBSOLETES:
                 /* Replaces operations are expressed in two steps: the first one is a SOLVER_TRANSACTION_OBSOLETES, with the name of
                  * the replacer package. The second one is a SOLVER_TRANSACTION_IGNORE, with the name of the replacee */
                 obs = pkgs->pkgs[i + 1];
                 ret = opkg_remove_pkg(obs);
-                if (ret)
-                    break;
+                if (ret) {
+                    err = -1;
+                    goto CLEANUP;
+                }
             case SOLVER_TRANSACTION_DOWNGRADE:
             case SOLVER_TRANSACTION_REINSTALL:
             case SOLVER_TRANSACTION_INSTALL:
@@ -916,6 +929,10 @@ static int libsolv_solver_execute_transaction(libsolv_solver_t *libsolv_solver)
                                  pkg->name, pkg->version, pkg->dest->name);
                 }
                 ret = opkg_install_pkg(pkg);
+                if (ret) {
+                    err = -1;
+                    goto CLEANUP;
+                }
                 break;
             case SOLVER_TRANSACTION_UPGRADE:
                 old = pkg_hash_fetch_installed_by_name(pkg->name);
@@ -944,14 +961,15 @@ static int libsolv_solver_execute_transaction(libsolv_solver_t *libsolv_solver)
                 }
 
                 ret = opkg_install_pkg(pkg);
+                if (ret) {
+                    err = -1;
+                    goto CLEANUP;
+                }
                 break;
             case SOLVER_TRANSACTION_IGNORE:
             default:
-                ret = 0;
                 break;
             }
-            if (ret)
-                err = -1;
         }
     }
 
